@@ -1,8 +1,5 @@
-from cvm.datatypes.enums                  import *
 from cvm.datatypes.tax_id                 import *
 from cvm.datatypes.currency               import *
-from cvm.datatypes.federated_state        import *
-from cvm.datatypes.country                import *
 from cvm.datatypes.security               import *
 from cvm.datatypes.account                import *
 from cvm.datatypes.statement              import *
@@ -19,4 +16,5 @@ from cvm.datatypes.controlling_interest   import *
 from cvm.datatypes.communication_channel  import *
 from cvm.datatypes.issuer                 import *
 from cvm.datatypes.shareholder_department import *
+from cvm.datatypes.capital_distribution   import *
 from cvm.datatypes.document               import *
\ No newline at end of file
