@@ -6,14 +6,14 @@ __all__ = [
     'Address'
 ]
 
-@dataclasses.dataclass(init=True, frozen=True)
+@dataclasses.dataclass(init=True)
 class Address:
     street: str
     complement: str
     district: str
     city: str
-    state: typing.Union[datatypes.FederatedState, str]
-    country: typing.Optional[datatypes.Country]
+    state: str
+    country: typing.Optional[str]
     postal_code: int
 
     __slots__ = (
