@@ -7,9 +7,9 @@ __all__ = [
     'TradingAdmission'
 ]
 
-@dataclasses.dataclass(init=True, frozen=True)
+@dataclasses.dataclass(init=True)
 class TradingAdmission:
-    foreign_country: typing.Optional[datatypes.Country]
+    foreign_country: typing.Optional[str]
     """(1.14) 'País estrangeiro em que os valores mobiliários são admitidos à negociação'"""
 
     admission_date: datetime.date
